"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var IdentifierInstance = /** @class */ (function () {
    function IdentifierInstance(opts) {
        this.buffer = (opts ? opts.buffer : undefined) || Buffer.alloc(0);
        this.alphabet =
            (opts ? opts.alphabet : undefined) || '0123456789ABCDEFGHIJKLMNOPQRSTVWXYZabcdefghijklmnopqrstvwxyz';
        this.minLength = (opts ? opts.minLength : undefined) || 0;
    }
    IdentifierInstance.prototype.fromBuffer = function (buffer) {
        var pos = 0;
        for (var i = 0; i <= buffer.length; i++) {
            if (buffer[i] !== 0x00) {
                pos = i;
                break;
            }
        }
        this.buffer = buffer.slice(pos);
        return this;
    };
    IdentifierInstance.prototype.toBuffer = function () {
        return this.buffer;
    };
    IdentifierInstance.prototype.fromHex = function (hex) {
        if (hex.length % 2)
            hex = '0' + hex;
        this.buffer = Buffer.from(hex, 'hex');
        return this;
    };
    IdentifierInstance.prototype.toHex = function () {
        return this.buffer.toString('hex');
    };
    IdentifierInstance.prototype.fromUUID = function (uuid) {
        return this.fromHex(uuid.replace(/-/g, ''));
    };
    IdentifierInstance.prototype.toUUID = function () {
        var hex = this.toHex().padStart(32, '0');
        var sections = [];
        sections.push(hex.substr(0, 8));
        sections.push(hex.substr(8, 4));
        sections.push(hex.substr(12, 4));
        sections.push(hex.substr(16, 4));
        sections.push(hex.substr(20));
        return sections.join('-');
    };
    IdentifierInstance.prototype.fromString = function (stringInput, alphabet) {
        return this.fromBuffer(require('base-x')(alphabet || this.alphabet).decode(stringInput));
    };
    IdentifierInstance.prototype.toString = function (arg1, arg2) {
        var optMinimumLength = typeof arg1 === 'number' ? arg1 : typeof arg2 === 'number' ? arg2 : this.minLength;
        var optAlphabet = typeof arg1 === 'string' ? arg1 : typeof arg2 === 'string' ? arg2 : this.alphabet;
        return require('base-x')(optAlphabet).encode(this.buffer).padStart(optMinimumLength, optAlphabet[0]);
    };
    IdentifierInstance.prototype.fromInt = function (numberInput) {
        return this.fromHex(numberInput.toString(16));
    };
    IdentifierInstance.prototype.toInt = function () {
        return parseInt(this.toHex(), 16);
    };
    IdentifierInstance.prototype.fromBigInt = function (bigInt) {
        return this.fromHex(bigInt.toString(16));
    };
    IdentifierInstance.prototype.toBigInt = function () {
        return BigInt('0x' + this.toHex());
    };
    IdentifierInstance.prototype.generateObjectId = function () {
        var time = new Date().getTime();
        var value = '';
        value += Math.floor(time / 1000)
            .toString(16)
            .padStart(8, '0');
        value += (time % 1000).toString(16).padStart(4, '0');
        while (value.length < 24)
            value += (Math.random() * 16 || 0).toString(16);
        return this.fromHex(value);
    };
    IdentifierInstance.prototype.generateBigInt = function () {
        var time = new Date().getTime();
        var value = '';
        value += Math.floor(time / 1000)
            .toString(16)
            .padStart(8, '0');
        value += (time % 1000).toString(16).padStart(4, '0');
        while (value.length < 16)
            value += (Math.random() * 16 || 0).toString(16);
        return this.fromHex(value);
    };
    IdentifierInstance.prototype.generateUUID = function (version) {
        switch (version) {
            case 1:
                return this.fromUUID(require('uuid').v1());
            case 4:
            default:
                return this.fromUUID(require('uuid').v4());
        }
    };
    return IdentifierInstance;
}());
exports.IdentifierInstance = IdentifierInstance;
function Identifier(opts) {
    return new IdentifierInstance(opts);
}
exports.Identifier = Identifier;
